/*
 * Socket client example
 */

#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <time.h>
#include "gpsd.h"

#define PORT            5555
#define SERVERHOST      "localhost"
#define MAXMSG		512

DBPOINT waypoint[MAX_WAYPOINTS];

void write_to_server(int filedes, char *msg)
{
	int nbytes;

	nbytes = write(filedes, msg, strlen(msg) + 1);
	if (nbytes < 0) {
		perror("write");
		exit(EXIT_FAILURE);
	}
}

int read_from_client(int filedes)
{
    char buffer[MAXMSG];
    int nbytes;

    nbytes = read(filedes, buffer, MAXMSG);
    if (nbytes < 0) {
        /* Read error. */
        perror("read");
        exit(EXIT_FAILURE);
    } else if (nbytes == 0)
        /* End-of-file. */
        return -1;
    else {
        /* Data read. */
//        fprintf(stderr, "client: got message: '%s'\n", buffer);
        return atoi(buffer);
    }
}


int data_read_from_client(int filedes, int count)
{
    int i;
    int nbytes;
    DBPOINT *dest = waypoint;
    char buffer[80];

    for (i = 0; i < count; i++) {

    sprintf(buffer, "WAY %d", i);
    write_to_server(filedes, buffer);
    
    nbytes = read(filedes, dest, sizeof(DBPOINT));
    if (nbytes < 0) {
        /* Read error. */
        perror("read");
        exit(EXIT_FAILURE);
    } else if (nbytes == 0)
        /* End-of-file. */
        return -1;
    else {
        /* Data read. */
//        fprintf(stderr, "read %d bytes\n", nbytes);
	dest++;
    }
    }
    return 0;
}


void init_sockaddr(struct sockaddr_in *name,
		   const char *hostname, uint16_t port)
{
	struct hostent *hostinfo;

	name->sin_family = AF_INET;
	name->sin_port = htons(port);
	hostinfo = gethostbyname(hostname);
	if (hostinfo == NULL) {
		fprintf(stderr, "Unknown host %s.\n", hostname);
		exit(EXIT_FAILURE);
	}
	name->sin_addr = *(struct in_addr *) hostinfo->h_addr;
}

int main(void)
{
	int i;
	int sock;
	int count;
	struct sockaddr_in servername;

	/* Create the socket. */
	sock = socket(PF_INET, SOCK_STREAM, 0);
	if (sock < 0) {
		perror("socket (client)");
		exit(EXIT_FAILURE);
	}

	/* Connect to the server. */
	init_sockaddr(&servername, SERVERHOST, PORT);
	if (connect(sock, (struct sockaddr *) &servername,
		    sizeof(servername)) < 0) {
		perror("connect (client)");
		exit(EXIT_FAILURE);
	}

	/* Send data to the server. */
	write_to_server(sock, "COUNT");
	count = read_from_client(sock);
	if (data_read_from_client(sock, count) != 0) {
		printf("failed to read all data\n");
		return 0;
	}

	/* Setup the filename */
	

for (i = 0; i < count; i++) {
	DBPOINT *p = &waypoint[i];
	struct tm *tm = gmtime(&p->timestamp);

    printf("%02d/%02d/%02d %02d:%02d:%02d (%.8f %.8f %d) %02x\n", 
	tm->tm_year+1900, tm->tm_mon, tm->tm_mday,
	tm->tm_hour, tm->tm_min, tm->tm_sec,
	p->latitude, p->longitude, p->altitude,
	p->mode_sv);
}

	close(sock);
	exit(EXIT_SUCCESS);
}
