/*
 * gpsd - monitoring daemon for Marconi GPS
 * Ralph Siemsen, August 2001
 */

#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <netdb.h>
#include "gpsparser.h"
#include "gpsd.h"

#define PORT    5555
#define MAXMSG  512

#define MESSAGE "Thanks i got it"

DBPOINT *waypoint;
int waypoint_count;

int make_socket(uint16_t port)
{
    int sock;
    struct sockaddr_in name;

    /* Create the socket. */
    sock = socket(PF_INET, SOCK_STREAM, 0);
    if (sock < 0) {
	perror("socket");
	exit(EXIT_FAILURE);
    }

    /* Give the socket a name. */
    name.sin_family = AF_INET;
    name.sin_port = htons(port);
    name.sin_addr.s_addr = htonl(INADDR_ANY);
    if (bind(sock, (struct sockaddr *) &name, sizeof(name)) < 0) {
	perror("bind");
	exit(EXIT_FAILURE);
    }

    return sock;
}

void write_to_server(int filedes, char *msg)
{
	int nbytes;

	nbytes = write(filedes, msg, strlen(msg) + 1);
	if (nbytes < 0) {
		perror("write");
		exit(EXIT_FAILURE);
	}
}

int read_from_client(int filedes)
{
    char buffer[MAXMSG];
    int nbytes;

    nbytes = read(filedes, buffer, MAXMSG);
    if (nbytes < 0) {
	/* Read error. */
	perror("read");
//	exit(EXIT_FAILURE);
	return -1;
    } else if (nbytes == 0)
	/* End-of-file. */
	return -1;
    else {
	/* Data read. */
	if (strcmp(buffer, "COUNT") == 0) {
		fprintf(stderr, "Server: got message: '%s'\n", buffer);
		sprintf(buffer, "%d", waypoint_count);
		write_to_server(filedes, buffer);
	}
	if (strncmp(buffer, "WAY", 3) == 0) {
		int count;
		sscanf(buffer, "WAY %d", &count);
		nbytes = write(filedes, &waypoint[count], sizeof(DBPOINT));
		if (nbytes < 0) {
			perror("write");
			exit(EXIT_FAILURE);
		}
	}
    }
    return 0;
}

int read_nmea(int filedes)
{
	char buffer[256];

	read(filedes, buffer, 256);
	printf("NMEA: %s\n", buffer);

	return 0;
}

int main(void)
{
    int i;
    int socket_fd, serial_fd, nmea_fd;
    fd_set active_fd_set, read_fd_set;
    struct sockaddr_in clientname;
    size_t size;
    int connected = 0;

    /* Allocate space to store waypoints */
    waypoint = malloc(MAX_WAYPOINTS * sizeof(DBPOINT));
    if (!waypoint) {
    	perror("malloc");
    	exit(1);
    }
    waypoint_count = 0;
    printf("Waypoint table allocated\n");

    /* Get GPS going on serial port */
    setserialport(GPS_PORT, GPS_SPEED);
    setserialport2(NMEA_PORT, NMEA_SPEED);
    printf("Serial ports initialized\n");
    serial_fd = serialfd;
    nmea_fd = nmeafd;
    cmc_tx_msg_63(63, "UGPS-000");
    cmc_tx_msg(0x94);
    printf("Init strings sent\n");

    /* Create the socket and set it up to accept connections. */
    socket_fd = make_socket(PORT);
    if (listen(socket_fd, 1) < 0) {
	perror("listen");
	exit(EXIT_FAILURE);
    }
    printf("Socket created\n");

    /* Initialize the set of active sockets. */
    FD_ZERO(&active_fd_set);
    FD_SET(socket_fd, &active_fd_set);
    FD_SET(serial_fd, &active_fd_set);
    FD_SET(nmea_fd, &active_fd_set);

printf("Entering loop\n");
    while (1) {
	/* Block until input arrives on one or more active sockets. */
	read_fd_set = active_fd_set;
	if (select(FD_SETSIZE, &read_fd_set, NULL, NULL, NULL) < 0) {
	    perror("select");
	    exit(EXIT_FAILURE);
	}

	/* Service all the sockets with input pending. */
	for (i = 0; i < FD_SETSIZE; ++i)
	    if (FD_ISSET(i, &read_fd_set)) {
	    	if (i == nmea_fd) {
	    		read_nmea(nmea_fd);
	    	} else if (i == serial_fd) {
	    		extern char nmeabuffer[];
	    		parse_rx_msg();
//	    		if (connected)
//	    			write_to_server(socket_fd, nmeabuffer);
	    	} else if (i == socket_fd) {
		    /* Connection request on original socket. */
		    int new;
		    size = sizeof(clientname);
		    new = accept(socket_fd, (struct sockaddr *)
				 &clientname, &size);
		    if (new < 0) {
			perror("accept");
			exit(EXIT_FAILURE);
		    }
		    fprintf(stderr,
			    "Server: connect from host %s, port %hd.\n",
			    inet_ntoa(clientname.sin_addr),
			    ntohs(clientname.sin_port));
		    FD_SET(new, &active_fd_set);
		    connected = 1;
		} else {
		    /* Data arriving on an already-connected socket. */
		    if (read_from_client(i) < 0) {
			close(i);
			FD_CLR(i, &active_fd_set);
			connected = 0;
		    }
		}
	    }
    }

    close(serial_fd);
    close(socket_fd);
}
